import React, { useState, useRef, useEffect, useMemo, useCallback } from "react";
import { useNavigate } from 'react-router-dom';
import {
    List,
    DatagridConfigurable,
    SearchInput,
    TopToolbar,
    SelectColumnsButton,
    EditButton,
    FilterButton,
    CreateButton,
    ExportButton,
    BulkDeleteButton,
    WrapperField,
    useRecordContext,
    useTranslate,
    useListContext,
    useCreatePath,
    TextField,
    NumberField,
    DateField,
    BooleanField,
    ReferenceField,
    TextInput,
    DateTimeInput,
    DateInput,
    SelectInput,
    NumberInput,
    ReferenceInput,
    ReferenceArrayInput,
    AutocompleteInput,
    DeleteButton,
} from 'react-admin';
import { Box, Typography, Card, Chip } from '@mui/material';
import { styled } from '@mui/material/styles';
import OperationRecordPanel from "./OperationRecordPanel";
import EmptyData from "@/page/components/EmptyData";
import MyExportButton from '@/page/components/MyExportButton';
import PageDrawer from "@/page/components/PageDrawer";
import MyField from "@/page/components/MyField";
import { PAGE_DRAWER_WIDTH, OPERATE_MODE, DEFAULT_PAGE_SIZE } from '@/config/setting';
import * as Common from '@/utils/common';
import rowSx from './rowSx';
import { format } from 'date-fns';
import OperationDetail from './OperationDetail'
import { width } from "@mui/system";

const StyledDatagrid = styled(DatagridConfigurable)(({ theme }) => ({
    '& .css-1vooibu-MuiSvgIcon-root': {
        height: '.9em'
    },
    '& .RaDatagrid-row': {
        cursor: 'auto'
    },
    '& .column-url': {
        maxWidth: '16em',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap',
    },
    '& .column-request': {
        maxWidth: '18em',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap',
    },
    '& .column-response': {
        maxWidth: '18em',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap',
    },
    '& .column-spendTime': {
        maxWidth: '9em',
    },
    '& .opt': {
        width: 200
    },
    '& .RaDatagrid-thead': {
        borderLeftColor: 'transparent',
        borderLeftWidth: 5,
        borderLeftStyle: 'solid',
    },
}));

const filters = [
    <SearchInput source="condition" alwaysOn />,
    <DateInput label='common.time.after' source="timeStart" alwaysOn />,
    <DateInput label='common.time.before' source="timeEnd" alwaysOn />,

    <TextInput source="namespace" label="table.field.operationRecord.namespace" />,
    <TextInput source="url" label="table.field.operationRecord.url" />,
    <TextInput source="appkey" label="table.field.operationRecord.appkey" />,
    <TextInput source="clientIp" label="table.field.operationRecord.clientIp" />,
    <TextInput source="request" label="table.field.operationRecord.request" />,
    <TextInput source="response" label="table.field.operationRecord.response" />,
    <NumberInput source="spendTime" label="table.field.operationRecord.spendTime" />,
    <SelectInput source="result" label="table.field.operationRecord.result"
        choices={[
            { id: 1, name: '成功' },
            { id: 0, name: '失败' },
        ]}
    />,
    <ReferenceInput source="userId" reference="user">
        <AutocompleteInput label="table.field.operationRecord.userId" optionText="nickname" filterToQuery={(val) => ({ nickname: val })} />
    </ReferenceInput>,

    <TextInput label="common.field.memo" source="memo" />,
]

const OperationRecordList = () => {
    const translate = useTranslate();

    const [createDialog, setCreateDialog] = useState(false);
    const [drawerVal, setDrawerVal] = useState(false);

    return (
        <Box display="flex">
            <List
                sx={{
                    flexGrow: 1,
                    transition: (theme) =>
                        theme.transitions.create(['all'], {
                            duration: theme.transitions.duration.enteringScreen,
                        }),
                    marginRight: drawerVal ? `${PAGE_DRAWER_WIDTH}px` : 0,
                }}
                title={"menu.operation"}
                empty={<EmptyData onClick={() => { setCreateDialog(true) }} />}
                filters={filters}
                sort={{ field: "create_time", order: "desc" }}
                actions={(
                    <TopToolbar>
                        <FilterButton />
                        <SelectColumnsButton preferenceKey='operationRecord' />
                        <MyExportButton />
                    </TopToolbar>
                )}
                perPage={DEFAULT_PAGE_SIZE}
            >
                <StyledDatagrid
                    preferenceKey='operationRecord'
                    bulkActionButtons={false}
                    rowClick={(id, resource, record) => {
                        setDrawerVal(!!drawerVal && drawerVal === record ? null : record);
                        return false;
                    }}
                    omit={['appkey', 'statusBool', 'err', 'updateTime', 'createTime', 'memo']}
                    rowSx={rowSx(drawerVal || null)}
                >
                    <NumberField source="id" />
                    <TextField source="namespace" label="table.field.operationRecord.namespace" />
                    <TextField source="url" label="table.field.operationRecord.url" />
                    <ReferenceField source="userId" label="table.field.operationRecord.userId" reference="user" link={false}>
                        <TextField source="nickname" />
                    </ReferenceField>
                    <TextField source="appkey" label="table.field.operationRecord.appkey" />
                    <FormattedTimestampField source="timestamp" label="table.field.operationRecord.timestamp" />
                    <NumberField source="spendTime" label="table.field.operationRecord.spendTime" sx={{ fontWeight: 'bold' }} />
                    <TextField source="request" label="table.field.operationRecord.request" sortable={false} hidden={!!drawerVal} />
                    <TextField source="response" label="table.field.operationRecord.response" sortable={false} hidden={!!drawerVal} />
                    <ResultField source="result" label="table.field.operationRecord.result" />
                    <TextField source="err" label="table.field.operationRecord.err" />
                    <TextField source="clientIp" label="table.field.operationRecord.clientIp" />
                    <DateField source="updateTime" label="common.field.updateTime" showTime />
                    <DateField source="createTime" label="common.field.createTime" showTime />
                    <BooleanField source="statusBool" label="common.field.status" sortable={false} />
                    <TextField source="memo" label="common.field.memo" sortable={false} />
                </StyledDatagrid>
            </List>
            <PageDrawer
                title='OperationRecord Detail'
                drawerVal={drawerVal}
                setDrawerVal={setDrawerVal}
            >
                <OperationDetail
                    operation={drawerVal}
                />
            </PageDrawer>
        </Box>
    )
}

const FormattedTimestampField = ({ source }) => {
    const record = useRecordContext();
    const val = record[source];
    if (!val) return null;
    const formattedDate = format(new Date(Number(val)), 'yyyy-MM-dd HH:mm:ss');
    return <span>{formattedDate}</span>;
};

const ResultField = ({ source }) => {
    const record = useRecordContext();
    const val = record[source];
    return (
        <>
            {val === 1
                ? (
                    <Chip label="success" color="success" variant="outlined" size="small" />
                )
                : (
                    <Chip label="error" color="error" variant="outlined" size="small" />
                )
            }

        </>
    )
}

export default OperationRecordList;
